//-----------------------------------------------------------------------------
/*!
   \file          main.c
   \brief         power supply logic for tube radio supply


   \implementation
    project       tube radio supply
    version       0.00
    copyright     Matthias Koelling

    generated     Matthias Koelling 31.03.2020
    review
    released

    compiler      AVR GCC
    target        ATTiny13A
   \endimplementation
*/
//-----------------------------------------------------------------------------

/* -- Includes ------------------------------------------------------------- */

#include <avr/io.h>
#include <avr/interrupt.h>
#include "ioport.h"
#include "uart.h"
#include "cb.h"

/* -- Makros --------------------------------------------------------------- */

/* -- Defines -------------------------------------------------------------- */

#define BUFFER_SIZE (200u)

/* -- Types ---------------------------------------------------------------- */

typedef struct
{
   uint8_t u8_control;
   uint8_t u8_data;
}
T_display_data;

/* -- Module Global Variables ---------------------------------------------- */

static T_circular_buffer mt_display_data_buffer;

/* -- Extern Global Variables ---------------------------------------------- */

/* -- Implementation Functions --------------------------------------------- */

//Index 1==0b0001 => 0b1000
//Index 7==0b0111 => 0b1110
//etc
static unsigned char lookup[16] = {
0x0, 0x8, 0x4, 0xc, 0x2, 0xa, 0x6, 0xe,
0x1, 0x9, 0x5, 0xd, 0x3, 0xb, 0x7, 0xf, };

uint8_t reverse(uint8_t n) {
   // Reverse the top and bottom nibble then swap them.
   return (lookup[n&0b1111] << 4) | lookup[n>>4];
}

// Detailed breakdown of the math
//  + lookup reverse of bottom nibble
//  |       + grab bottom nibble
//  |       |        + move bottom result into top nibble
//  |       |        |     + combine the bottom and top results
//  |       |        |     | + lookup reverse of top nibble
//  |       |        |     | |       + grab top nibble
//  V       V        V     V V       V
// (lookup[n&0b1111] << 4) | lookup[n>>4]

//-----------------------------------------------------------------------------
/*!
   \brief      main

               hardware init and run

   \author     Koelling
   \date       31.03.2020

   \param[in]  none
   \param[out] none
   \return     none
*/
//-----------------------------------------------------------------------------
int16_t main (void)
{
   T_display_data t_display_data;
   char ac_start_text[] = {'H','e','l','l','o',' ','W','o','r','l','d',0};
   uint8_t i = 0;
   uint8_t u8_state = 0;

   MCUCR |= 0x10;
   EICRA = 0x02;
   EIMSK = 0x01;

   ioport_init ();
   uart_init (9600);
   cb_init (&mt_display_data_buffer, BUFFER_SIZE, sizeof (T_display_data));

    do
    {
       //uart_send (0xAA);
       //if (uart_send (ac_start_text[i] == true))
       if (uart_send (ac_start_text[i]) == true)
       {
          i++;
       }
   }
   while (ac_start_text[i] != 0);

   while (1)
   {

      switch (u8_state)
      {
      case 0:
         if (cb_pop_front (&mt_display_data_buffer, &t_display_data) == true)
         {
            if (uart_send (t_display_data.u8_control) == true)
            {
               u8_state++;
            }
         }
         break;

      case 1:
         if (uart_send (t_display_data.u8_data) == true)
         {
            u8_state++;
         }
         break;

      case 2:
         if (uart_send ('\n') == true)
         {
            u8_state = 0;
         }
         break;

      default:
         break;
      }

   }
   return 0;
}

ISR (INT0_vect)
{
   T_display_data t_display_data;

   t_display_data.u8_control = PIND >> 3;
   t_display_data.u8_data = PINC;
   cb_push_back (&mt_display_data_buffer, &t_display_data);
}

// EOF
